#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <stdlib.h>
#include <portbits.h>
#include "param.h"
#include "global.h"
#include "lcd.h"
#include "uart.h"

//***************************************************************************
//*																			*
//*					LCD Controller fr Dualscan 640x480 LCD					*
//*																			*
//*								mit UART									*
//*																			*
//*								Version 1.00								*
//*																			*
//*								 by Benedikt								*
//*																			*
//*						Email:	benedikt83 t gmx.net						*
//*																			*
//***************************************************************************
//*																			*
//*	Die Software darf frei kopiert und verndert werden, solange sie nicht	*
//*	ohne meine Erlaubnis fr kommerzielle Zwecke eingesetzt wird.			*
//*																			*
//***************************************************************************

#define UART_BAUD_RATE		   115200 

int main(void)
{	PORTD=1;
	DDRB=255;
	DDRD=254;
	DDRE=255;
	PORTA=255;
	PORTC=255;

	uart_init(UART_BAUD(UART_BAUD_RATE, F_CPU),UART_BAUD_U2X(UART_BAUD_RATE, F_CPU));
	lcd_init();									// LCD Controller initialisieren
	
	unsigned char c;
	unsigned short xpos=0, ypos=0;

	for (;;)
	{	c = uart_getchar();
		if (c>31)				//Display Text
		{	lcd_writechar(xpos/4,ypos,c);
			xpos+=TEXT_X;
			if (xpos>=XSIZE)
			{	xpos=0;
				ypos+=TEXT_Y;
				if (ypos>=YSIZE)
					ypos=0;
			}
		}
		else if (c==1)			// Cursor Home
		{	xpos=0;
			ypos=0;
		}
		else if (c==2)			// LCD off
		{	ENABLE_VLCD=0;
		}
		else if (c==3)			// LCD on
		{	ENABLE_VLCD=1;
		}
		else if (c==8)			// Backspace
		{	xpos-=TEXT_X;
			if (xpos>=32768)
			{	xpos=XSIZE-TEXT_X;
				ypos-=TEXT_Y;
				if (ypos>=32768)
					ypos=YSIZE-TEXT_Y;
			}
			lcd_writechar(xpos/4,ypos,32);
		}
		else if (c==9)			// Set Pixel
		{	CONVERTW x,y;
			x.b[0]=uart_getchar();
			x.b[1]=uart_getchar();
			y.b[0]=uart_getchar();
			y.b[1]=uart_getchar();
			lcd_setpixel(x.w,y.w);
		}
		else if (c==10)        // Line Feed
		{	ypos+=TEXT_Y;
			if (ypos>=YSIZE) 
				ypos=0;
		}
		else if (c==11)        // Delete in place
		{	lcd_writechar(xpos/4,ypos,32);
		}
		else if (c==12)        // Form Feed
		{	xpos=0;
			ypos=0;
			lcd_clear();
		}
		else if (c==13)			// Carriage Return
		{	xpos=0;
		}
		else if (c==14)			// Set Brightness
		{	c=uart_getchar();
		}
		else if (c==15)			// Set Contrast
		{	c=uart_getchar();
			if (c>100)
				c=100;
			OCR1B=c*2+c/2;
		}
		else if (c==16)			//Load BMP
		{	CONVERTW y1,ys;
			unsigned short y;
			unsigned char x1,xs,x,c;
			if (uart_getchar()==0xAA)
			{	x1=uart_getchar();
    			y1.b[0]=uart_getchar();
    			y1.b[1]=uart_getchar();
    			xs=uart_getchar();
    			ys.b[0]=uart_getchar();
    			ys.b[1]=uart_getchar();
    			for (y=0; y<ys.w; y++)
    			{	for (x=0; x<xs; x++)
    				{	c=uart_getchar();
    					lcd_writebyte(x*2+x1,y+y1.w,c);
    				}
				}
			}
		}
		else if (c==17)        // Set Cursor Position (Column and Row)
		{	xpos=uart_getchar()*TEXT_X;
			ypos=uart_getchar()*TEXT_Y;
			if (xpos>=XSIZE)
				xpos=0;
			if (ypos>=YSIZE)
				ypos=0;
		}
		else if (c==18)			// Draw Line
		{	CONVERTW x1,y1,x2,y2;
			x1.b[0]=uart_getchar();
			x1.b[1]=uart_getchar();
			y1.b[0]=uart_getchar();
			y1.b[1]=uart_getchar();
			x2.b[0]=uart_getchar();
			x2.b[1]=uart_getchar();
			y2.b[0]=uart_getchar();
			y2.b[1]=uart_getchar();
			lcd_line (x1.w,y1.w,x2.w,y2.w);
		}
		else if (c==21)			// Scroll
		{	uart_getchar();
			uart_getchar();
		}
		else if (c==22)			// Enable Scroll
		{	uart_getchar();
			uart_getchar();
			uart_getchar();
		}
		else if (c==25)			// Get CGRAM
		{	c=uart_getchar();
			c&=7;
			for (unsigned char i=0; i<8; i++)
				cgram[c][i]=uart_getchar();
		}
		else if (c==28)			// Large Number
		{	uart_getchar();
			uart_getchar();
			uart_getchar();
		}
		else if (c==30)			// Send directly
		{	c=uart_getchar();
			if (c==0)
				lcd_writecgram(xpos/4,ypos,uart_getchar());
			else
				lcd_writechar(xpos/4,ypos,uart_getchar());
   			xpos+=TEXT_X;
   			if (xpos>=XSIZE)
   			{	xpos=0;
   				ypos+=TEXT_Y;
   				if (ypos>=YSIZE)
   					ypos=0;
			}
		}
	}
}


