#ifndef UART_H
#define UART_H

// UART library with receive/transmit circular buffers based on Interrupt UART library from Peter Fleury <pfleury@gmx.ch>   http://jump.to/fleury

#define ENABLE_RX				// Empfangsroutinen verwenden
//#define ENABLE_TX				// Senderoutinen verwenden
#define SIMPLE_UART				// nur einfache, platzparende Sende und Empfangsroutinen verwenden
//#define DISABLE_TXBUF			// Sendepuffer abschalten
#define RX_COUNT				// Anzahl der Bytes im Empfangsbuffer zhlen (z.B. fr CTS)
#define USE_CTS					// CTS Handshaking
//#define NO_UART_INT			// kein Interrupt, nur Polling

#define CTS		PORTD_1			// CTS Pin (muss per Hand auf Ausgang gesetzt werden !!!)

#define FIFO_LEVEL	75			// max FIFO Level ab dem CTS deaktiviert wird (in %)

#define UART_RX_BUFFER_SIZE 256	// UART0 RX Buffer
#define UART_TX_BUFFER_SIZE 192	// UART0 TX Buffer

#define UART1_RX_BUFFER_SIZE 32	// UART1 RX Buffer
#define UART1_TX_BUFFER_SIZE 32	// UART1 TX Buffer


// macros for baudrate calculation
#define DEFABS(val)									(((val)<0)?-val:val)
#define UART_BAUD_ERROR(baudRate,xtalCpu,Divider)	(((((signed long)(xtalCpu)/((Divider)*16L))-(signed long)(baudRate))*1000L)/(signed long)(baudRate))
#define UART_BAUD_SELECT(baudRate,xtalCpu) 			(((signed long)(xtalCpu)+(baudRate)*8L)/((signed long)(baudRate)*16L)-1)
#define UART_BAUD2_ERROR(baudRate,xtalCpu,Divider)	(((((signed long)(xtalCpu)/((Divider)*8L))-(signed long)(baudRate))*1000L)/(signed long)(baudRate))
#define UART_BAUD2_SELECT(baudRate,xtalCpu) 		((signed long)((xtalCpu)+(signed long)(baudRate)*4L)/((signed long)(baudRate)*8L)-1)
#define UART_BAUD_U2X(baudRate,xtalCpu) 			((DEFABS(UART_BAUD_ERROR(baudRate,xtalCpu,UART_BAUD_SELECT(baudRate,xtalCpu)+1))>15)?(DEFABS(UART_BAUD_ERROR(baudRate,xtalCpu,UART_BAUD_SELECT(baudRate,xtalCpu)+1))>DEFABS(UART_BAUD2_ERROR(baudRate,xtalCpu,UART_BAUD2_SELECT(baudRate,xtalCpu)+1))?1:0):0)
#define UART_BAUD(baudRate,xtalCpu) 				(UART_BAUD_U2X(baudRate,xtalCpu)?UART_BAUD2_SELECT(baudRate,xtalCpu):UART_BAUD_SELECT(baudRate,xtalCpu))
#define UART_BAUDERROR(baudRate,xtalCpu)			(UART_BAUD_U2X(baudRate,xtalCpu)?DEFABS(UART_BAUD2_ERROR(baudRate,xtalCpu,UART_BAUD2_SELECT(baudRate,xtalCpu)+1)):DEFABS(UART_BAUD_ERROR(baudRate,xtalCpu,UART_BAUD_SELECT(baudRate,xtalCpu)+1)))

// high byte error return code of uart_getc()
#define UART_FRAME_ERROR      0x0800              /* Framing Error by UART       */
#define UART_OVERRUN_ERROR    0x0400              /* Overrun condition by UART   */
#define UART_BUFFER_OVERFLOW  0x0200              /* receive ringbuffer overflow */
#define UART_NO_DATA          0x0100              /* no receive data available   */

extern void uart_init(unsigned int baudrate, unsigned char u2x);

extern unsigned int uart_getc(void);
extern void uart_putc(unsigned char data);
extern void uart_puts(const char *s );
extern void uart_puts_P(const char *s );
extern unsigned char uart_getchar(void);
#if ((defined RX_COUNT)&&(UART_RX_BUFFER_SIZE>=256))
extern unsigned short uart_data(void);
#else
extern unsigned char uart_data(void);
#endif

extern void uart1_init(unsigned int baudrate);
extern unsigned int uart1_getc(void);
extern void uart1_putc(unsigned char data);
extern void uart1_puts(const char *s );
extern void uart1_puts_P(const char *s );


#endif // UART_H 

